( cd usr/lib ; rm -rf libiso9660++.so.0 )
( cd usr/lib ; ln -sf libiso9660++.so.0.0.0 libiso9660++.so.0 )
( cd usr/lib ; rm -rf libiso9660++.so )
( cd usr/lib ; ln -sf libiso9660++.so.0.0.0 libiso9660++.so )
( cd usr/lib ; rm -rf libcdio++.so.0 )
( cd usr/lib ; ln -sf libcdio++.so.0.0.0 libcdio++.so.0 )
( cd usr/lib ; rm -rf libcdio++.so )
( cd usr/lib ; ln -sf libcdio++.so.0.0.0 libcdio++.so )
( cd usr/lib ; rm -rf libcdio.so.7 )
( cd usr/lib ; ln -sf libcdio.so.7.0.0 libcdio.so.7 )
( cd usr/lib ; rm -rf libcdio.so )
( cd usr/lib ; ln -sf libcdio.so.7.0.0 libcdio.so )
( cd usr/lib ; rm -rf libiso9660.so.5 )
( cd usr/lib ; ln -sf libiso9660.so.5.0.0 libiso9660.so.5 )
( cd usr/lib ; rm -rf libiso9660.so )
( cd usr/lib ; ln -sf libiso9660.so.5.0.0 libiso9660.so )
( cd usr/lib ; rm -rf libcdio_cdda.so.0 )
( cd usr/lib ; ln -sf libcdio_cdda.so.0.0.2 libcdio_cdda.so.0 )
( cd usr/lib ; rm -rf libcdio_cdda.so )
( cd usr/lib ; ln -sf libcdio_cdda.so.0.0.2 libcdio_cdda.so )
( cd usr/lib ; rm -rf libcdio_paranoia.so.0 )
( cd usr/lib ; ln -sf libcdio_paranoia.so.0.0.2 libcdio_paranoia.so.0 )
( cd usr/lib ; rm -rf libcdio_paranoia.so )
( cd usr/lib ; ln -sf libcdio_paranoia.so.0.0.2 libcdio_paranoia.so )
( cd usr/lib ; rm -rf libudf.so.0 )
( cd usr/lib ; ln -sf libudf.so.0.0.0 libudf.so.0 )
( cd usr/lib ; rm -rf libudf.so )
( cd usr/lib ; ln -sf libudf.so.0.0.0 libudf.so )
